#!/usr/bin/python3
# Copyright (c) 2005 Novell, Inc.
# All Rights Reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General Public
# License as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail,
# you may find current contact information at www.novell.com
#
#
#  Author:  Novell, Inc.
#  Workfile:  clstrlibss.py
#  Contents:  Secret store class
#

import pickle
import base64
import copy
import os
import string
import ldap
import syslog
import inspect
import sys
import subprocess

ldap.set_option(ldap.OPT_NETWORK_TIMEOUT,8)
ldap.set_option(ldap.OPT_TIMEOUT,16)
certPath = subprocess.Popen(['sed', '-n', 's/TLS_CERT //p', '/etc/openldap/ldap.conf'], stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()[0]
certPath = bytes.decode(certPath).rstrip()
if certPath != '':
    ldap.set_option(ldap.OPT_X_TLS_CACERTFILE, certPath)
    ldap.set_option(ldap.OPT_X_TLS_NEWCTX, 0)

fname = '/etc/opt/novell/ncs/clstrlib.conf'

class data:
    "A class to store clstrlib's secret store data"
    ldapUrl = ""
    adminDn = ""
    adminPw = ""
    clusterDn = ""
    clusterIp = ""


def obfuscate_string(inStr):
    try:
        return base64.encodestring(str.encode('base64' + inStr))
    except:
        syslog.syslog(syslog.LOG_INFO, f" {inspect.currentframe().f_code.co_name} failed: " + str(sys.exc_info()[0]))
        return inStr

def deobfuscate_string(inStr):
    if isinstance(inStr, str):
        tinStr = str.encode(inStr)
    else:
        tinStr = inStr
    try:
        clearStr = base64.decodestring(tinStr)
        clearStr = bytes.decode(clearStr)
        if clearStr.startswith('base64'):
            return clearStr[6:]
        else:
            syslog.syslog(syslog.LOG_INFO, f" {inspect.currentframe().f_code.co_name}1 failed: " + str(sys.exc_info()[0]))
            return inStr
    except:
        syslog.syslog(syslog.LOG_INFO, f" {inspect.currentframe().f_code.co_name}2 failed: " + str(sys.exc_info()[0]))
        return inStr

def is_string_obfuscated(inStr):
    return deobfuscate_string(inStr) != inStr

def is_obfuscated(c):
    if is_string_obfuscated(c.adminDn) and is_string_obfuscated(c.adminPw):
        return True
    return False

# All elements of c must be in clear text when passed to obfuscate
def obfuscate(c):
    r = copy.copy(c)
    r.adminDn = obfuscate_string(r.adminDn)
    r.adminPw = obfuscate_string(r.adminPw)
    return r

def deobfuscate(c):
    r = copy.copy(c)
    r.adminDn = deobfuscate_string(r.adminDn)
    r.adminPw = deobfuscate_string(r.adminPw)
    return r

def casa_available():
    return True

def save(c, filename=fname, move_to_casa=False):
    d = copy.copy(c)
    try:
        os.putenv("SECRET_ID", "NovellClusterServices.Novell")
        os.putenv("SECRET_USERNAME", d.adminDn)
        os.putenv("SECRET_PASSWORD", d.adminPw)
        if os.system("/opt/novell/ncs/bin/ncs-casautil -a") == 0:
            d.adminPw = ""
            d.adminDn = ""
        elif move_to_casa:
            return
    except:
        if move_to_casa:
            return

    out = obfuscate(d)

    f = open(filename, 'wb+')
    pickle.dump(out, f)
    f.close()

def load(filename=fname):
    f = open(filename, 'rb')
    c = data()
    c = pickle.load(f)
    f.close()

    save_needed = not is_obfuscated(c)
    c = deobfuscate(c)
    try:
        if (c.adminPw == "" or c.adminDn == ""):
            os.putenv("SECRET_ID", "NovellClusterServices.Novell")
            child=os.popen("/opt/novell/ncs/bin/ncs-casautil -d")
            out = child.read()
            child.close()
            if c.adminDn == "":
                c.adminDn = out.split('\n')[1]
            if c.adminPw == "":
                c.adminPw = out.split('\n')[2]
        else:
            save(c, filename, not save_needed)
            save_needed = False
    except:
        pass

    try:
        ld = ldap.initialize(c.ldapUrl)
    except:
        if save_needed:
            save(c, filename)
        return c

    try:
        ld.simple_bind_s( c.adminDn, c.adminPw )
    except:
        if save_needed:
            save(c, filename)
        return c

    try:
        c.clusterDn = ld.search_s(c.clusterDn, ldap.SCOPE_BASE)[0][0]
    except:
        try:
            child=os.popen( "grep -o '^n4u\\.nds\\.server-name=.*$' /etc/opt/novell/eDirectory/conf/nds.conf | cut -d'=' -f2" )
            out = child.read()
            child.close()
            serverName = "cn=" + out.split('\n')[0]
            child=os.popen( "grep -o '^n4u\\.nds\\.server-context=.*' /etc/opt/novell/eDirectory/conf/nds.conf | sed 's/^n4u\\.nds\\.server-context=//'" )
            out = child.read()
            child.close()
            res = ld.search_s( serverName + "," + out.split('\n')[0].replace( '.', ',' ), ldap.SCOPE_BASE, serverName, [ "nCSNetWareCluster" ] )
            if res[0][1].has_key( "nCSNetWareCluster" ):
                c.clusterDn = res[0][1]["nCSNetWareCluster"][0]
        except:
            pass

    try:
        ld.unbind_s()
    except:
        pass

    if save_needed:
        save(c, filename)
    return c
