#!/usr/bin/perl
#
# Copyright (c) 2013 Novell, Inc.
# All Rights Reserved.
#  Author:  Novell, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General Public
# License as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail,
# you may find current contact information at www.novell.com
#
#  Workfile:  dotoutparser.pl
#  Contents:  Utility to parse *.out files
#

use strict;

my $verbose = 0;
my $error = 1;
my $printLN = "\\d+: ";
my $pCount = 0;

my $search_key1 = "exit_on_error";
my $search_key2 = "ignore_error";
my @search_keys = ( $search_key1, $search_key2 );

sub ProcessRecord
{
    my ($record, $startDate, ( $key1, $key2 )) = @_;
    my $processed;
    my $failed = 0;

    $record =~ s/\r\n/\n/mg;

    if( $record =~ /^${printLN}\+ $key1 .*$/m )
    {
        $processed = $`;
    }
    else
    {
        $processed = $record;
    }

    while ($record =~ /^${printLN}\+ $key1 .*$/mg )
    {
        my $entry = $';
        my $left = '';

        $processed .= $&;

        if( $entry =~ /^${printLN}\+ $key1 .*$/m )
        {
            $entry = $`;
        }

        if( $entry =~ /^${printLN}\+ $key2 .*$/m )
        {
            $left = $& . $';
            $entry = $`;
        }

        if( $entry !~ /^${printLN}\+ (local )?rc=0\n/m )
        {
            $failed = 1;

            if( $entry  =~ /^.*\n.*\n/m )
            {
                $processed .= $&;
                $entry = $';
            }
            if( $entry =~ /^${printLN}\+ (local )?rc=\d+\n/m )
            {
                $left = $' . $left;
                $entry = $` . $&;
            }
            $entry =~ s/^(${printLN})(.*)$/\1\+ dotoutparser \2/mg;
        }

        $processed .= $entry . $left;
    }

    if( 0 != $error && 0 == $failed ) { return };

    $pCount++;
    if( 0 == $verbose )
    {
        my $filtered = '';

        $processed =~ s/^${printLN}\+\++ .*\n//mg;
        while ( $processed =~ /^${printLN}\+ .*\n/mg )
        {
           $filtered .= $&;
        }

        $filtered =~ s/^(${printLN}\+ return 0\n)?${printLN}\+ (local )?rc=\d+\n${printLN}\+ date\n.*\n//mg;
        $filtered =~ s/^${printLN}\+ date\n.*\n//mg;
        $filtered =~ s/^${printLN}\+ eval .*\n//mg;
        $filtered =~ s/^${printLN}\+ \w+=.*\n//mg;
        $processed = $filtered;
    }
    elsif( 1 == $verbose )
    {
        my $filtered = '';

        $processed =~ s/^${printLN}\+\++ .*\n//mg;
        while ( $processed =~ /^${printLN}\+ .*\n/mg )
        {
            $filtered .= $&;
        }

        $filtered =~ s/^(${printLN}\+ return 0\n)?${printLN}\+ (local )?rc=\d+\n${printLN}\+ date\n.*\n//mg;
        $filtered =~ s/^${printLN}\+ date\n.*\n//mg;
        $filtered =~ s/^${printLN}\+ eval .*\n//mg;
        $processed = $filtered;
    }
    elsif( 2 == $verbose )
    {
        my $filtered = '';

        $processed =~ s/^${printLN}\+\+\++ .*\n//mg;
        while ( $processed =~ /^${printLN}\++ .*\n/mg )
        {
            $filtered .= $&;
        }

        $filtered =~ s/^(${printLN}\+ return 0\n)?${printLN}\+ (local )?rc=\d+\n${printLN}\+ date\n.*\n//mg;
        $filtered =~ s/^${printLN}\+ eval .*\n//mg;
        $processed = $filtered;
    }
    elsif( 3 == $verbose )
    {
        my $filtered = '';

        $processed =~ s/^${printLN}\+\+\+\++ .*\n//mg;
        while ( $processed =~ /^${printLN}\++ .*\n/mg )
        {
           $filtered .= $&;
        }
        $processed = $filtered;
    }

    $processed =~ s/^(${printLN})\+ dotoutparser /\1/mg;
    print "$startDate (#$pCount):\n$processed\n";
}

sub print_usage
{
   print STDERR "Usage: $0 [-a] [-n] [-v|-vv|-vvv|-vvvv] file_name|-\n";
   print STDERR "  [-a] print all records.\n";
   print STDERR "  [-l] analyse the output of load script.\n";
   print STDERR "  [-u] analyse the output of unload script.\n";
   print STDERR "  [-m] analyse the output of monitor script.\n";
   print STDERR "  [-n] do not add line numbers.\n";
   print STDERR "  [-i] print ignored errors.\n";
   print STDERR "  [-v|-vv|-vvv|-vvvv] specify verbose level.\n";
   print STDERR "  file_name|- specify input file or standard input.\n";
   print STDERR "Example:\n";
   print STDERR "  /opt/novell/ncs/bin/dotoutparser.pl -a /var/opt/novell/log/ncs/AUTO_POOL_12_SERVER.load.out\n";
   print STDERR "  /opt/novell/ncs/bin/dotoutparser.pl -u -n -i AUTO_POOL_12_SERVER -vv\n";
}

my $file_name = '';
my $file_type = '.load.out';

foreach my $arg (@ARGV)
{
    if ( '--help' eq $arg || '-h' eq $arg || '-?' eq $arg )
    {
        &print_usage();
        exit 0;
    }

    if ( '-a' eq $arg ){ $error = 0; }
    elsif ( '-n' eq $arg ){ $printLN = ''; }
    elsif ( '-i' eq $arg ){ @search_keys = reverse @search_keys; }
    elsif ( '-' eq $arg ){ $file_name = "&STDIN"; }
    elsif ( '-l' eq $arg ){ $file_type = ".load.out"; }
    elsif ( '-m' eq $arg ){ $file_type = ".monitor.out"; }
    elsif ( '-u' eq $arg ){ $file_type = ".unload.out"; }
    elsif ( "-v" eq $arg){ $verbose = 1; }
    elsif ( "-vv" eq $arg ){ $verbose = 2; }
    elsif ( "-vvv" eq $arg ){ $verbose = 3; }
    elsif ( "-vvvv" eq $arg ){ $verbose = 4; }
    else { $file_name = $arg; }
}

if ( '' eq $file_name )
{
    &print_usage();
    exit 1;
}

if( 0 == open( input_file, "<$file_name" ))
{
    if( $file_name !~ /\.load/ && $file_name !~ /\.unload/ && $file_name !~ /\.monitor/)
    {
        $file_name =~ s/\.out$//;
        $file_name .= $file_type;
    }
    if( $file_name !~ /\.out$/ )
    {
        $file_name .= ".out";
    }

    my $pos = rindex(  $file_name, "/" );

    if( -1 != $pos )
    {
        $file_name = substr( $file_name, $pos + 1 );
    }

    $file_name = "/var/opt/novell/log/ncs/" . $file_name;

    #print "file_name=$file_name.\n";

    if( 0 == open( input_file, "<$file_name" ))
    {
        print STDERR "Failed to open input file.\n";
        exit 2;
    }
}

my $content = '';
my $stamp;

while ( <input_file> )
{
    if ( $_ =~ /^CRM: (.*)$/ )
    {
        if ( '' ne $content )
        {
            &ProcessRecord( $content, $stamp, @search_keys );
        }
        $stamp = $+;
        $content = '';
    }
    else
    {
        if( '' ne $printLN )
        {
            $content .= "$.: ";
        }
        $content .= $_;
    }
}

close( input_file );

&ProcessRecord( $content, $stamp, @search_keys );
exit 0;

