#!/usr/bin/python3

#
# Copyright (c) 2005 Novell, Inc.
# All Rights Reserved.
#  Author:  Novell, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General Public
# License as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail,
# you may find current contact information at www.novell.com
#
#  Workfile:  ncs-configd.py
#  Contents:  Userspace cluster configuration daemon
#

import os
import sys
import syslog
import imp
import ldap
import clstrlib
import clstrlibss
import ncstempl
import time
import string
import fcntl
import subprocess
#############################################################################
#
# Declarations
#

# clstrlibfile = "/tmp/clstrlib"
clstrlibfile = "/proc/ncs/clstrlib"

#############################################################################
#
# Initialization
#

# open syslog
syslog.openlog("ncs-configd")

# export NCSCONFIGD=1 enables debug
debug = os.getenv("NCSCONFIGD", "disabled")

os.environ["PATH"] ="/sbin" + os.pathsep + os.environ["PATH"]
# load the secret store
ss = clstrlibss.load()

# initialize LDAP options
ldap.set_option(ldap.OPT_TIMELIMIT, 8)
ldap.set_option(ldap.OPT_NETWORK_TIMEOUT, 16)
certPath = subprocess.Popen(['sed', '-n', 's/TLS_CERT //p', '/etc/openldap/ldap.conf'], stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()[0]
certPath = bytes.decode(certPath).rstrip()
if certPath != '':
    ldap.set_option(ldap.OPT_X_TLS_CACERTFILE, certPath)
    ldap.set_option(ldap.OPT_X_TLS_NEWCTX, 0)
#############################################################################
#
# Function: importClstrlibCall
#
#   Purpose: import code calling clstrlib, held in request
#
def importClstrlibCall(request):
    module = imp.new_module("clstrlibcall")
    request = "import clstrlib\nargs = " + request
    exec(request,  module.__dict__)
    return module

#############################################################################
#
# Function: executeClstrlibRpc
#
#   Purpose: execute clstrlib RPC request, return response
#
def executeClstrlibRpc(request):

    if (debug != "disabled"):
        syslog.syslog(syslog.LOG_DEBUG, "request: " + request)

    if (request.startswith("NWCLSTR_UtcToCmaDateAndTime")):
        utcSeconds = request.split("=")[1]
        tm = time.localtime(int(utcSeconds))
        response = str(tm[0]) + "=" + str(tm[1]-1) + "=" + str(tm[2]) + "=" + str(tm[3]) + "=" + str(tm[4]) + "=" + str(tm[5])

    elif (request.startswith("NWCLSTR_Abend")):
        args = request.split("=")
        syslog.syslog(syslog.LOG_EMERG, "NWCLSTR_Abend: " + args[1])
        response = os.system(args[2])

    elif (request.startswith("NWCLSTR_Rexec")):
        command = request.split("=", 1)[1]
        response = os.system(command)

    elif (request.startswith("NWCLSTR_PipeExec")):
        command = request.split("=", 1)[1]
        result = os.popen(command)
        response = result.read()
        result.close()

    else:
        for index in range( 3 ):
            try:
                #syslog.syslog(syslog.LOG_INFO, f"URL: {ss.ldapUrl}; DN: {ss.adminDn}; PW: {ss.adminPw}")
                ld = ldap.initialize(ss.ldapUrl)
                ld.simple_bind_s(ss.adminDn, ss.adminPw)

                try:
                    m = importClstrlibCall(request)

                    try:
                        response = m.args[0](ld, ss.clusterDn, m.args)
                    except:
                        response = -97
                        syslog.syslog(syslog.LOG_ERR, request + " failed: " + str(sys.exc_info()))

                except:
                    response = -98
                    syslog.syslog(syslog.LOG_ERR, "importClstrlibCall failed: " + str(sys.exc_info()))

                try:
                    ld.unbind_s()
                except:
                    pass
                #syslog.syslog(syslog.LOG_INFO, "Break from function call")

                break

            except:
                if index < 2:
                    ss = clstrlibss.load()
                else:
                    response = -99
                    syslog.syslog(syslog.LOG_ERR, "LDAP bind failed: " + str(sys.exc_info()[0]))

    if (debug != "disabled"):
        syslog.syslog(syslog.LOG_DEBUG, "response: " + str(response))

    #syslog.syslog(syslog.LOG_INFO, "response: " + str(response))
    return response

#############################################################################
#
# Main
#

if (len(sys.argv) == 2):

# initialize cluster configuration

    if (sys.argv[1] == "-init"):

        ld = ldap.initialize(ss.ldapUrl)
        ld.simple_bind_s(ss.adminDn, ss.adminPw)

        clstrlib.updateClusterConfig(ld, ss.clusterDn)

        ld.unbind_s()

# request from args, response to stdout

    else:
        #syslog.syslog(syslog.LOG_INFO, "executeClstrlibRpc: ")
        response = str(executeClstrlibRpc(sys.argv[1]))
        #syslog.syslog(syslog.LOG_INFO, "executeClstrlibRpc: response " +response)
        sys.stdout.write(response)
        #syslog.syslog(syslog.LOG_INFO, "executeClstrlibRpc: response sent")
        #sys.stdout.write(str(executeClstrlibRpc(sys.argv[1])))

else:

# run cluster configuration daemon

    clstrlib.daemonize()

    try:
        lock = os.fdopen( os.open( "/var/run/ncs/lock.clstrlibd", os.O_RDWR | os.O_CREAT), "r+")
    except IOError as e:
        syslog.syslog(syslog.LOG_ERR, "Failed to open lock file: " + str(e) )
        sys.exit( -1 )

    try:
        fcntl.lockf(lock, fcntl.LOCK_EX | fcntl.LOCK_NB )
    except IOError as e:
        pid = ""
        try:
            pid = lock.readline().rstrip()
        except IOError as et:
            syslog.syslog(syslog.LOG_ERR, str(et))

        syslog.syslog(syslog.LOG_ERR, "Failed to obtain the exclusive lock: " + str(e))
        if pid:
            syslog.syslog(syslog.LOG_ERR, "Locked by " + pid )
        lock.close()
        sys.exit( -2 )

    try:
        lock.write( "%05d\n" % os.getpid())
        lock.flush()
    except:
        pass

    while True:

        # read request
        f = open(clstrlibfile, "r")
        request1 = f.read()
        request = request1.replace('"', '\"')
        f.close()
        #syslog.syslog(syslog.LOG_INFO, f"  - read \"{request}\" from {clstrlibfile}")
        if (request.startswith("exit")):
            syslog.syslog(syslog.LOG_DEBUG, "recieved exit")
            break

        if (request.startswith("NCSCONF_WriteConfFile")):
            content = request.split( ",", 2 )
            response = str( len( content[2] ))
            for i in range( 1, 5 ):
                try:
                    fd = open( content[1], 'w' )
                    fd.write( content[2] )
                    fd.close()
                    break;
                except:
                    time.sleep( 1 )
                    response = "0"
        elif (request.startswith("[clstrlib.NWCLSTR_ReadConfFile")):
            content = request.split( "\"", 2 )
            for i in range( 1, 5 ):
                try:
                    fd = open( content[1], 'r' )
                    response = fd.read()
                    fd.close()
                    break;
                except:
                    time.sleep( 1 )
                    response = "0"
        elif (request.startswith("NCSCONF_STONITH_KILL")):
            content = request.split( ",", 2 )
            response = 0
            if (os.path.isfile( content[1] )):
                try:
                    response = os.fork()
                    if ( 0 == response ):
                        clstrlib.daemonize()
                        os.system( content[1] + " " + content[2] + " | logger -t NCS_STONITH" )
                        os._exit(0)
                    else:
                        os.waitpid(response, 0)
                except:
                    response = os.system( content[1] + " " + content[2] + " | logger -t NCS_STONITH_NO_FORK" )
        elif (request.startswith("[clstrlib.NWCLSTR_FileExists")):
            response = 0
            content = request.split( "\"", 2 )
            if (os.path.isfile( content[1] )):
                response = 1
        else:
            response = executeClstrlibRpc(request)
        if ("" == response):
            response = '\0'

        # write response
        f = open(clstrlibfile, "w")
        f.write(str(response))
        f.close()

    syslog.syslog(syslog.LOG_DEBUG, "exiting")

    try:
        fcntl.lockf(lock, fcntl.LOCK_UN )
    except Exception as e:
        syslog.syslog(syslog.LOG_ERR, "Failed to release the lock: " + str(e))
    lock.close()

syslog.closelog()
