#!/usr/bin/python3

#
# Copyright (c) 2005 Novell, Inc.
# All Rights Reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General Public
# License as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail,
# you may find current contact information at www.novell.com
#
#  Author:  Novell, Inc.
#  Workfile:  ncstempl.py
#  Contents:  Userspace cluster configuration daemon
#

import os
import sys
import getopt
import ldap
import string
import fcntl
import struct
import clstrlib
import clstrlibss
import subprocess

import xml.sax
from xml.sax.handler import *
import xml.dom.minidom

ldap.set_option(ldap.OPT_NETWORK_TIMEOUT,8)
ldap.set_option(ldap.OPT_TIMEOUT,16)
certPath = subprocess.Popen(['sed', '-n', 's/TLS_CERT //p', '/etc/openldap/ldap.conf'], stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()[0]
certPath = bytes.decode(certPath).rstrip()
if certPath != '':
    ldap.set_option(ldap.OPT_X_TLS_CACERTFILE, certPath)
    ldap.set_option(ldap.OPT_X_TLS_NEWCTX, 0)
#############################################################################
#
# Class: TriggerResourceUpdate
#
#   Purpose:
#
def TriggerResourceUpdate( list ):
    adminFile = open( "/admin/Novell/Cluster/ClusterConfig.cmd", "r+")
    adminFile.write( "<virtualIO><datastream name=\"command\"/></virtualIO>" )
    adminFile.seek( 0, 0 )
    adminFile.write( "<ncsRequest><clusterConfig><setClusterInfo><priorityInfo>"  + list + "</priorityInfo></setClusterInfo></clusterConfig></ncsRequest>" )
    adminFile.seek( 0, 0 )
    print (adminFile.read())
    adminFile.close()

#############################################################################
#
# Class: resourceConfigXmlHandler
#
#   Purpose:
#
class resourceConfigXmlHandler(xml.sax.handler.ContentHandler):

    def __init__(self):
        self.results = []

    def startElement(self, name, attrs):
       self.value = ""
       if( "dsml:entry" == name ):
           self.name = ""
           self.match = 0
       elif( "dsml:attr" == name and "name" in attrs ):
           self.attr = attrs.getValue( "name" )

    def endElement(self, name):
        if( "dsml:entry" == name ):
            if( 0 != self.match and "" != self.name ):
                self.results.append( self.name )
        elif( "dsml:oc-value" == name ):
            self.value.strip()
            if( "nCSVolumeResource" == self.value or "nCSClusterResource" == self.value ):
                self.match = 1
        elif( "dsml:attr" == name ):
            if( "cn" == self.attr ):
                self.name = self.value.strip()

    def characters(self, ch):
        self.value += ch

    def getResult(self):
        return self.results

#############################################################################
#
# Class: resourceRuntimeXmlHandler
#
#   Purpose:
#
class resourceRuntimeXmlHandler(xml.sax.handler.ContentHandler):

    def __init__(self):
        self.member = 1
        self.results = []

    def startElement(self, name, attrs):
       self.value = ""

    def endElement(self, name):
        if( "name" == name ):
            self.results.append( self.value.strip())
        elif( "failed" == name ):
            self.member = 0

    def characters(self, ch):
        self.value += ch

    def getResult(self):
        return ( self.member, self.results )

#############################################################################
#
# Function: getResourceConfigNamesAndRevisions
#
#   Purpose:
#
def parseXMLFile( fileName, xmlHandler ):

    parser = xml.sax.make_parser()
    parser.setFeature(xml.sax.handler.feature_external_ges, False)
    parser.setContentHandler(xmlHandler)
    parser.parse( fileName )
    return xmlHandler.getResult()

def NotInTheOtherList( listA, listB ):
    aNotinB = []
    bNotinA = []
    indexA = 0
    indexB = 0
    lenA = len( listA )
    lenB = len( listB )

    while 1:
        if( indexA < lenA ):
            if( indexB < lenB ):
                if( listA[indexA] < listB[indexB] ):
                    aNotinB.append( listA[indexA] )
                    indexA += 1
                elif( listA[indexA] > listB[indexB] ):
                    bNotinA.append( listB[indexB] )
                    indexB += 1
                else:
                    indexA += 1
                    indexB += 1
            else:
                while( indexA < lenA ):
                    aNotinB.append( listA[indexA] )
                    indexA += 1
                break
        else:
            while( indexB < lenB ):
                bNotinA.append( listB[indexB] )
                indexB += 1
            break

    return ( aNotinB, bNotinA )
#
# Main
#
def main(argv=None):
    master_IP_resource_name = "Master_IP_Address_Resource"

    try:
        ss = clstrlibss.load()
    except Exception as e:
        sys.stderr.write( "Failed to load CASA credentials: " )
        sys.stderr.write( str(e) + "\n" )
        return -1

    try:
        ld = ldap.initialize(ss.ldapUrl)
        ld.simple_bind_s(ss.adminDn, ss.adminPw)
    except Exception as e:
        sys.stderr.write( "Failed to bind to LDAP/eDir: " )
        sys.stderr.write( str(e) + "\n" )
        return -2

    try:
        eDirRes = ld.search_s( ss.clusterDn, ldap.SCOPE_ONELEVEL, "(|(objectClass=nCSClusterResource)(objectClass=nCSVolumeResource))", [ "nCSRevision" ] )
    except Exception as e:
        sys.stderr.write( "Failed to search cluster object (" + ss.clusterDn + ") for resources: " )
        sys.stderr.write( str(e) + "\n" )
        ld.unbind_s()
        return -3

    eDirNames = []
    eDirRevisions = {}

    for entry in eDirRes:
        name = clstrlib.objectNameFromLdapDn( entry[0] )
        eDirNames.append( name )
        eDirRevisions[name] = entry[1]["nCSRevision"][0]

    eDirNames.sort()

    print(eDirNames)
    print(eDirRevisions)
    for entry in eDirNames:
        print ("Directory - " + entry + ": " + bytes.decode(eDirRevisions[entry]))

    configRes = parseXMLFile( "/var/opt/novell/ncs/resources.xml", resourceConfigXmlHandler())

    configRes.sort()
    for item in configRes:
        print ("Config - name='" + item + "'")

    ( a, b ) = NotInTheOtherList( eDirNames, configRes )
    if a or b:
        print ("Updating cluster configuration.")
        clstrlib.updateClusterConfig( ld, ss.clusterDn )

    try:
        eDirStream = ld.search_s( ss.clusterDn, ldap.SCOPE_BASE, "cn=" + clstrlib.objectNameFromLdapDn( ss.clusterDn ), [ "nCSCRMResourcePriority" ] )
    except Exception as e:
        sys.stderr.write( "Failed to retrieve resource priority: " )
        sys.stderr.write( str(e) + "\n" )
        ld.unbind_s()
        return -4

    print ("Directory:\n" + bytes.decode(eDirStream[0][1]["nCSCRMResourcePriority"][0]))

    revision = 0
    numRes = 0
    streamedRes = []
    streamedNames = []
    minPriority = 50000
    for line in (bytes.decode(eDirStream[0][1]["nCSCRMResourcePriority"][0])).split("\n"):
        items = line.split( "=", 3 )
        if( "RESOURCE" == items[0] ):
            streamedRes.append( (items[1], items[2]))
            streamedNames.append( items[1] )
            priority = int ( items[2] )
            if( priority < minPriority ):
                minPriority = priority
        elif ( "REVISION" == items[0] ):
            revision = int( items[1] )
        elif ( "NUMRESOURCES" == items[0] ):
            numRes = int( items[1] )

    streamedNames.sort()

    if( 0 >= minPriority ):
        deltaPriority = 0
    elif ( 100 >= minPriority ):
        deltaPriority = 1
    else:
        deltaPriority = 10

    print ("revision = " + str( revision ) + " (" + str( minPriority ) + ")")
    actualResources = len( eDirNames )
    print ("number of resources = " + str( numRes ) + " : " + str( actualResources ))
    for entry in streamedNames:
        print ("name = '" + entry + "'")

    needUpdate = 0
    (a, b) = NotInTheOtherList( eDirNames, streamedNames )
    if a:
        revision += 1
        needUpdate = 1
        for entry in a:
            print ("need to add '" + entry + "' to resources priority list.")
    if b:
        revision += 1
        needUpdate = 1
        for entry in b:
            print ("need to remove '" + entry + "' from resources priority list.")
    if not a and not b and actualResources != numRes:
        revision += 1
        needUpdate = 1
        print ("need to correct number of resources from " + str( numRes ) + " to " + str( actualResources ) + ".")

    priorityList ="<resourceList>"
    stream = "REVISION=" + str( revision ) + "\nNUMRESOURCES=" + str( actualResources ) + "\n"
    if b:
        for item in b:
            idx = 0
            for entry in streamedRes:
                if item == entry[0]:
                    del streamedRes[idx]
                    break
                idx += 1

    for entry in streamedRes:
        stream += "RESOURCE=" + entry[0] + "=" + entry[1] + "\n"
        if master_IP_resource_name != entry[0]:
            priorityList +="<name>" + entry[0] + "</name>"

    if a:
        for entry in a:
            minPriority -= deltaPriority
            if( 0 > minPriority):
                minPriority = 0
            stream += "RESOURCE=" + entry + "=" + str( minPriority ) + "\n"
            if master_IP_resource_name != entry:
                priorityList +="<name>" + entry + "</name>"

    priorityList +="</resourceList>"

    print ("Generated Resource Priority Stream: ")

    print (stream)
    print
    print (priorityList)

    if needUpdate:
        try:
            attrs = []
            attrs.append( (ldap.MOD_REPLACE, "nCSCRMResourcePriority", str.encode(stream, encoding = 'utf-8')) )
            print ("Modifying resource priority stream.")
            ld.modify_s(ss.clusterDn, attrs)
            clstrlib.createResourcePriorityConfFile(ld, ss.clusterDn)
        except Exception as e:
            sys.stderr.write( "nCSCRMResourcePriority update failed.\n" )
            sys.stderr.write( str(e) + "\n" )

    ld.unbind_s()

    ( member, runtimeRes ) = parseXMLFile( "/admin/Novell/Cluster/ResourceState.xml", resourceRuntimeXmlHandler())

    if( 0 != member ):
        runtimeRes.sort()
        for item in runtimeRes:
            print ("Runtime = name='" + item + "'")

    ( a, b ) = NotInTheOtherList( eDirNames, runtimeRes )
    if a:
        for entry in a:
            print ("need to add '" + entry + "' to runtime resources.")
            procFile = open( "/proc/ncs/cluster", "w" )
            procFile.write( "RESOURCES " + entry + " =" + str( eDirRevisions[entry] ))
            procFile.close()
    if b:
        for entry in b:
            print ("need to remove '" + entry + "' from runtime resources.")
        TriggerResourceUpdate( priorityList )

    return 0

if __name__ == "__main__":
    info = ( 0, 0, 0, 0 )
    try:
        lock = open( "/proc/ncs/vll", 'r' )
    except IOError as e:
        sys.stderr.write( "NCS is not running: " + str(e) + "\n" )
        sys.exit( -1 )
    try:
        readBuf = lock.read()
        if isinstance(readBuf, str):
           readBuf = str.encode(readBuf)
        info = struct.unpack( "IIII", readBuf)
        lock.close()
    except IOError as e:
        sys.stderr.write( "Failed to retrieve cluster membership: " + str(e) + "\n" )
        sys.exit( -2 )

    print ("Membership information: " + str( info ))
    if 0 == info[0]:
        print ("Not a member of the cluster.")
        sys.exit( -3 )
    if info[1] != info[3]:
        print ("This node is not the master.")
        sys.exit( -4 )

    try:
        lock = os.fdopen( os.open( "/var/run/ncs/lock.repair", os.O_RDWR | os.O_CREAT), "r+")
    except IOError as e:
        sys.stderr.write( "Failed to open lock file: " + str(e) + "\n" )
        sys.exit( -5 )

    try:
        sys.stdout.write( "Acquiring lock ... " )
        fcntl.lockf(lock, fcntl.LOCK_EX )
        print ("done")
    except IOError as e:
        pid = ""
        try:
            pid = lock.readline().rstrip()
        except IOError as et:
            sys.stderr.write( str(et))

        syslog.syslog(syslog.LOG_ERR, "Failed to obtain the exclusive lock: " + str(e) + "\n" )
        if pid:
            sys.stderr.write( "Locked by " + pid )
        lock.close()
        sys.exit( -6 )
    try:
        lock.write( "%05d\n" % os.getpid())
        lock.flush()
    except:
        pass

    rc = main(sys.argv)

    try:
        print ("Releasing lock")
        fcntl.lockf(lock, fcntl.LOCK_UN )
    except Exception as e:
        sys.stderr.write( "Failed to release the lock: " + str(e) + "\n" )
    lock.close()

    if 0 != rc :
        rc -= 6
    sys.exit( rc )

