#!/usr/bin/perl

use strict;

my $hostname               = "$ENV{HOSTNAME}";
my $masterNode             = `grep masterNode /admin/Novell/Cluster/NodeState.xml 2>/dev/null`;
my $workMode               = 0;
my $rc                     = 0;
my $scripts                = "";

sub getResourceScripts
{
    my $resource_name  = $_[0];
    my $rc  = 0;
    my $loadScript  = "";
    my $response ;

    if( open VIRTFILE, "+</admin//Novell/Cluster/ClusterConfig.cmd" )
    {
        my $command = "<virtualIO><datastream name=\"command\"/></virtualIO>";
        $command .= "<ncsRequest><clusterConfig><getResourceInfo>";
        $command .= "<name>$resource_name</name><typeOfInfo><scriptInfo/></typeOfInfo>";
        $command .= "</getResourceInfo></clusterConfig></ncsRequest>";
        syswrite VIRTFILE, $command, length($command);

        seek VIRTFILE, 0, 0;
        local $/;
        $response = <VIRTFILE>;
        close VIRTFILE;

        if( $response =~ m/<description>success<\/description>/ )
        {
            if( $response =~ m/<scriptInfo>\n.*<\/scriptInfo>\n/sm )
            {
                $response = $&;
            }
            else
            {
                $rc = 13;
            }
        }
        else
        {
            $rc = 12;
        }
    }
    else
    {
        $rc = 11;
    }

    return( $rc, $response );
}

sub setResourceScripts
{
    my ( $resource_name, $scripts ) = @_;
    my $rc = 0;
    my $response;

    if( open VIRTFILE, "+</admin//Novell/Cluster/ClusterConfig.cmd" )
    {
        my $command = "<virtualIO><datastream name=\"command\"/></virtualIO>";
        $command .= "<ncsRequest><clusterConfig><setResourceInfo>";
        $command .= "<name>$resource_name</name>";
        $command .= $scripts;
        $command .= "</setResourceInfo></clusterConfig></ncsRequest>";
        syswrite VIRTFILE, $command, length($command);

        seek VIRTFILE, 0, 0;
        local $/;
        $response = <VIRTFILE>;
        close VIRTFILE;

        if( $response !~ m/<description>success<\/description>/ )
        {
            $rc = 22;
        }
    }
    else
    {
        $rc = 21;
    }

    return( $rc, $response );
}

sub print_usage_and_exit
{
    print STDERR "Usage: $0 [--read|write] resource_name\n";
    print STDERR "--read: retrieve script information of the resource and print it to standard output\n";
    print STDERR "--write: gather script information from standard input and update the resource\n";
    print STDERR "--help|-h|-?: print this help page\n";
    exit $_[0];
}

if( $#ARGV < 0 )
{
    &print_usage_and_exit( 3 );
}

if( "--help" eq @ARGV[0] || "-?" eq @ARGV[0] || "-h" eq @ARGV[0] )
{
    &print_usage_and_exit( 0 );
}

if( $masterNode =~ /^<masterNode>(.+)<\/masterNode>$/ )
{
    $masterNode = $+;
}
else
{
    print STDERR "This node ($hostname) is not a member of the cluster.\n";
    exit 1;
}

if( $masterNode ne $hostname )
{
    print STDERR "This node ($hostname) is not the master ($masterNode) of the cluster.\n";
    exit 2;
}

if( "--write" eq @ARGV[0] )
{
    $workMode = 1;
    shift @ARGV;
}
else
{
    if ( "--read" eq @ARGV[0] )
    {
        $workMode = 0;
        shift @ARGV;
    }
}

if( 0 == $workMode )
{
    ( $rc, $scripts ) = getResourceScripts( $ARGV[0] );
    if( 0 != $rc )
    {
        print STDERR "Failed to retrieve scripts of resource '$ARGV[0]' ($rc).\n";
    }
    else
    {
        print $scripts;
    }
}
else
{
    while(<STDIN>)
    {
        $scripts .= "$_";
    }

    $scripts =~ s/\r\n/\n/g;
    ( $rc, $scripts ) = setResourceScripts( $ARGV[0], $scripts );
    if( 0 != $rc )
    {
        print STDERR "Failed to update scripts of resource '$ARGV[0]' ($rc).\n$scripts\n";
    }
}

exit $rc;

