#!/usr/bin/python3

#
# Copyright (c) 2005 Novell, Inc.
# All Rights Reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General Public
# License as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail,
# you may find current contact information at www.novell.com
#
#  Author:  Novell, Inc.
#  Workfile:  ncstempl.py
#  Contents:  Userspace cluster configuration daemon
#
import base64
import clstrlib
import clstrlibss
#import dircache
#import dsml
import getopt
import ldap
import os
import string
import sys
import subprocess

import xml.sax
from xml.sax.handler import *
import xml.dom.minidom

default_path='/opt/novell/ncs/templates'
if(sys.maxsize > 2 **32):
    sys.path.append(r'/opt/novell/ncs/lib64/')
else:
    sys.path.append(r'/opt/novell/ncs/lib/')
import dsml

ldap.set_option(ldap.OPT_NETWORK_TIMEOUT,8)
ldap.set_option(ldap.OPT_TIMEOUT,16)
certPath = subprocess.Popen(['sed', '-n', 's/TLS_CERT //p', '/etc/openldap/ldap.conf'], stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()[0]
certPath = bytes.decode(certPath).rstrip()
if certPath != '':
    ldap.set_option(ldap.OPT_X_TLS_CACERTFILE, certPath)
    ldap.set_option(ldap.OPT_X_TLS_NEWCTX, 0)
#############################################################################
#
# Function: addResourceTemplate
#
#   Purpose:
#
def addResourceTemplate(ld, clusterDn, fileName):
    ret,attrs = getTemplateAttributes(fileName)

    if ret == False:
        return False

    templateName = findAttribute(attrs, 'cn')
    if isinstance(templateName, bytes):
        templateName = bytes.decode(templateName)
    templateDn = "cn=" + templateName + "," + clusterDn

    try:
        ld.add_s(templateDn, attrs)
    except:
        return False
    return True


#############################################################################
#
# Function: addResourceTemplate
#
#   Purpose:
#
def addResourceTemplates(ld, clusterDn, path=default_path):
    templateFiles = os.listdir(path)
    for fileName in templateFiles:
        addResourceTemplate(ld, clusterDn, os.path.join(path, fileName))


#############################################################################
#
# Function: addResourceTemplate
#
#   Purpose:
#
def updateResourceTemplate(ld, clusterDn, fileName):
    ret,attrs = getTemplateAttributes(fileName)

    if ret == False:
        return False

    templateName = findAttribute(attrs, 'cn')
    if isinstance(templateName, bytes):
        templateName = bytes.decode(templateName)
    templateDn = "cn=" + templateName + "," + clusterDn

    try:
        ld.delete_s(templateDn)
    except:
        pass

    try:
        ld.add_s(templateDn, attrs)
    except:
        return False
    return True


#############################################################################
#
# Function: addResourceTemplate
#
#   Purpose:
#
def updateResourceTemplates(ld, clusterDn, path=default_path):
    templateFiles = os.listdir(path)
    for fileName in templateFiles:
        updateResourceTemplate(ld, clusterDn, os.path.join(path, fileName))


#############################################################################
#
# Class: templateXmlHandler
#
#   Purpose:
#
class templateXmlHandler(xml.sax.handler.ContentHandler):

    def __init__(self):
        self.attrList = []
        self.dsmlTag = None
        self.dsmlName = None
        self.dsmlValue = ""
        self.dsmlEncode = None

    def getResult(self):
        return self.attrList

    def startElement(self, name, attrs):
        self.dsmlTag = name
        if (self.dsmlTag == "dsml:attr"):
            self.dsmlName = attrs.getValue("name")
        elif (self.dsmlTag == "dsml:value"):
            try:
                self.dsmlEncode = attrs.getValue("encoding")
            except:
                self.dsmlEncode = None
                pass


    def endElement(self, name):
        if (name != "dsml:value"):
            return

        value = ""
        if self.dsmlEncode == 'base64':
            value = base64.b64decode(self.dsmlValue)
        else:
            tmpValue = self.dsmlValue
            value = str(tmpValue.strip().replace(u'\u2019',r"'"))

        self.attrList.append( (str(self.dsmlName), value) )
        self.dsmlValue = ""

    def characters(self, ch):
        if (self.dsmlTag != "dsml:value"):
            return
        self.dsmlValue = self.dsmlValue + ch


#############################################################################
#
# Function: getResourceAttribute
#
#   Purpose:
#
def findAttribute(attrList, attrName):
    for attr in attrList:
        if attr[0] == attrName:
            return attr[1]
    return ''


#############################################################################
#
# Function: getResourceAttribute
#
#   Purpose:
#
def getTemplateAttributes(fileName):

    value = None
    try:
        parser = xml.sax.make_parser()
        parser.setFeature(xml.sax.handler.feature_external_ges, False)
        handler = templateXmlHandler()
        parser.setContentHandler(handler)
        parser.parse(fileName)
        value = handler.getResult()
    except:
        pass

    if (value != None):
        for i, attr in enumerate(value):
            value[i] = attr[0], bytes(str(attr[1]), "utf-8")
        return (True, value)
    else:
        return (False, value)


#############################################################################
#
# Function: createResourcesXmlFile
#
#   Purpose:
#
def createTemplateXmlFiles(ld, clusterDn, path=default_path):

    res = ld.search_s(clusterDn, ldap.SCOPE_ONELEVEL, "objectClass=nCSResourceTemplate" )

    for resource in res:
        xml = open(os.path.join(path, resource[1]["cn"][0]+'.xml'), "w+")
        d = dsml.DSMLWriter(xml, ["nCSCRMPreferredNodes"], "", "  ")
        d.writeHeader()
        d.writeRecord(resource[0], resource[1])
        d.writeFooter()
        xml.close()



#############################################################################
#
# Main
#
def main(argv=None):

    path = default_path
    ss = clstrlibss.load()
    ld = ldap.initialize(ss.ldapUrl)
    ld.simple_bind_s(ss.adminDn, ss.adminPw)
    opts, args = getopt.getopt(argv[1:], 'a:u:AUd:g', ['add=', 'update=', 'add_templates', 'update_templates', 'dir=', 'directory=', 'get_templates'])

    action = None
    for o, a in opts:
        if o in ('-a', '--add'):
            addResourceTemplate(ld, ss.clusterDn, a)
        if o in ('-u', '--update'):
            updateResourceTemplate(ld, ss.clusterDn, a)
        if o in ('-d', '--dir', '--directory'):
            path = a
        if o in ('-U', '--update_templates'):
            action = updateResourceTemplates
        if o in ('-A', '--add_templates'):
            action = addResourceTemplates
        if o in ('-g', '--get_templates'):
            action = createTemplateXmlFiles

    if action != None:
        action(ld, ss.clusterDn, path)

    ld.unbind_s()

if __name__ == "__main__":
    sys.exit(main(sys.argv))
