#!/usr/bin/python3

#*********************************************************************
#
#  Copyright (c) 2005-2007 Novell, Inc.
#  All Rights Reserved.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of version 2 of the GNU General Public
#  License as published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, contact Novell, Inc.
#
#  To contact Novell about this file by physical or electronic mail,
#  you may find current contact information at www.novell.com
#
#   Author:  Novell, Inc.
#   Workfile:  sbd.py
#   Contents:  NetWare style wrapper around sbdutil
#
#*********************************************************************

import os
#import popen2
import string
import sys
import tty
import termios
import subprocess

binDir = "/opt/novell/ncs/bin"
sbdutil = binDir + "/sbdutil"

#############################################################################
#
# usage
# print the usage for this module
#
def usage(command):
    print ('usage: ' + command + ' [install][view][view all]\n', file=sys.stderr)

#############################################################################
#
# deviceList
# Helper function for getDevice
# returns a list of valid devices that sbd may be installed on
#
def deviceList():
    cmd = [sbdutil," -l"]
    out = subprocess.Popen(cmd)

    devList = []
    for dev in out[0]:
        devList.append(dev[:-1])
    return devList

#############################################################################
#
# getDevice
# Helper function for install.
# Lists valid devices and alows the user to choose one.
#
def getDevice():
    devList = deviceList()
    i = 0
    while i < len(devList):
        print (i, "-> Device Number for:",  devList[i])
        i += 1

    dev = None
    while dev is None:
        dev = input("Select device to install Persistent Cluster Map (SBD), Device Number = ")
        if dev < 0 or dev >= len(devList):
            print ("Invalid device number")
            dev = None
    return devList[dev]

#############################################################################
#
# getch
# a quick and dirty implementation of C's getch
#
def getch():
    fd = sys.stdin.fileno()
    old_settings = termios.tcgetattr(fd)
    tty.setraw(sys.stdin.fileno())
    ch = sys.stdin.read(1)
    termios.tcsetattr(fd, termios.TCSANOW, old_settings)
    return ch

#############################################################################
#
# install
# Copy of NetWare's SBD INSTALL command.
#
def install():
    cmd = [sbdutil, " -f"]
    f = subprocess.Popen(cmd)
    ret = f.wait()
    if (ret == 0):
        print ("SBD Partition already exists with the name " + f.fromchild.readline()[:-1])
    else:
        print ("Do you want to create Mirrored SBD Partition(y/n): ")
        ch = ''
        while ch.lower() != 'y' and ch.lower() != 'n':
            ch = getch()

        mirrored = False
        if ch.lower() == 'y':
            mirrored = True

        sbdDevices = []
        if not mirrored:
            sbdDevices.append(getDevice())
        else:
            print ("Please select the First device for Mirrored SBD Partition.")
            sbdDevices.append(getDevice())
            print ("Please select the Second device for Mirrored SBD Partition.")
            sbdDevices.append(getDevice())

        cmd = sbdutil + " -c"
        for dev in sbdDevices:
            cmd += " -d " + dev

        ret = os.system(cmd)
        if ret == 0:
            print ("SBD partition was created")
        elif mirrored:
            print ("Unable to create Mirrored SBD partition")
        else:
            print ("Unable to create SBD Partition")
            #"Unable to create SBD Partition with the name %s"
            #"Failed to create SBD Partition: Insufficient Space"
            #"SBD Partition already exists with the name %s"

#############################################################################
#
# view
# Copy of NetWare's SBD VIEW
#
def view():
    print ('Not implemented. Use "sbd view all".')

#############################################################################
#
# viewAll
# Copy of NetWare's SBD VIEW ALL
#
def viewAll():
    cmd = sbdutil + " -v"
    return os.system(cmd)

#############################################################################
#
# Main
#
def main(argv=None):
    if argv is None:
        argv = sys.argv

    if (len(argv) == 2):
        if (argv[1].lower() == "install"):
            return install()
        elif (argv[1].lower() == "view"):
            return view()
    elif (len(argv) == 3):
        if (argv[1].lower() == "view" and argv[2].lower() == "all"):
            return viewAll()

    return usage(argv[0])


if __name__ == "__main__":
    sys.exit(main())
