#!/usr/bin/python3

# Copyright (C) 2008 Novell, Inc. All rights reserved.
#
# This work is subject to U.S. and international copyright laws and treaties.
# No part of this work may be used, practiced, performed, copied, distributed,
# revised, modified, translated, abridged, condensed, expanded, collected,
# compiled, linked, recast, transformed or adapted without the prior written
# consent of Novell, Inc. Any use or exploitation of this work without authorization
# could subject the perpetrator to criminal and civil liability.

import os
import sys
import getopt
import ldap
import string
import syslog
import subprocess

sys.path.append('/opt/novell/ncs/bin')
import clstrlib
import clstrlibss

ldap.set_option(ldap.OPT_NETWORK_TIMEOUT,8)
ldap.set_option(ldap.OPT_TIMEOUT,16)
certPath = subprocess.Popen(['sed', '-n', 's/TLS_CERT //p', '/etc/openldap/ldap.conf'], stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()[0]
certPath = bytes.decode(certPath).rstrip()
if certPath != '':
    ldap.set_option(ldap.OPT_X_TLS_CACERTFILE, certPath)
    ldap.set_option(ldap.OPT_X_TLS_NEWCTX, 0)

def enableNativeVolume( ld, clusterDn, ncpServerDn, containerDn, volumeDn, cluster_name ):
    if not volumeDn.lower().startswith( "cn=" ):
        ## try to locate the object
        volDn = "cn=" + volumeDn + "," + containerDn
        try:
            res = ld.search_s(volDn, ldap.SCOPE_BASE)
        except:
            volDn = ""

        if not volDn:
            res = ld.search_s(containerDn, ldap.SCOPE_SUBTREE, "objectClass=volume", ["hostResourceName"] )
            for vol in res:
                if vol[1].has_key( "hostResourceName" ):
                    if vol[1]["hostResourceName"][0].lower() == volumeDn.lower():
                        volDn = vol[0]
    else:
        volDn = volumeDn

    if volDn:
        try:
            res = ld.search_s(volDn, ldap.SCOPE_BASE)
        except:
            sys.stderr.write( "Failed to find object: " + volDn + ".\n" );
            return 1
    else:
        return 12

    if not res:
        return 11

    if isinstance(volDn, bytes):
        tmp = clstrlib.containerFromLdapDn(bytes.decode(volDn) )
    else:
        tmp = clstrlib.containerFromLdapDn(volDn)

    if isinstance(tmp, bytes):
        tmp = bytes.decode(tmp)
    if isinstance(cluster_name, bytes):
        cluster_name =  bytes.decode(cluster_name)
    if isinstance(res[0][1]["hostResourceName"][0], bytes):
        tmp2 = bytes.decode(res[0][1]["hostResourceName"][0])
    else:
        tmp2 = res[0][1]["hostResourceName"][0]
    newDn = "cn=" + cluster_name + "_" + tmp2 + "," + tmp

    try:
        ld.rename_s( volDn, newDn )
    except:
        sys.stderr.write( "Failed to rename object: " + volDn + ".\n" );
        return 2

    print ("Object '" + volDn + "' renamed to '" + newDn + "'.")
    print ("The volume name you need to use in the scripts is: " + tmp)

    attrs = []
    attrs.append( (ldap.MOD_ADD, "nCSVolumes", str.encode(newDn)) )
    try:
        ld.modify_s( ncpServerDn, attrs )
    except:
        syslog.syslog(syslog.LOG_INFO, f" Failed to link volume " + str(sys.exc_info()[0]))
        sys.stderr.write( "Failed to link volume '" + newDn + "' to server '" + ncpServerDn + "'.\n" )
        return 3

    attrs = []
    attrs.append( (ldap.MOD_REPLACE, "hostServer", str.encode(ncpServerDn)) )
    try:
        ld.modify_s( newDn, attrs )
    except:
        sys.stderr.write( "Failed to link server '" + ncpServerDn + "' to volumer '" + newDn + "'.\n" )
        return 4

    print ("NCP server '" + ncpServerDn + "' and volume '" + newDn + "' are linked with each other.")

#############################################################################
#
# Function: main
#
# Returns: Nothing.
#
# Comments:
#
def main():
    rc = 0
    absoluteName = False
    createNCPSserver = False
    deleteNCPServer = False
    ncp_server_name = ''
    ncp_ip_address = ''
    ncp_volumes = []

    try:
        opts, args = getopt.getopt(sys.argv[1:], 'a: c: d: i: v:', [])
    except getopt.GetoptError as exc:
        print (exc.msg)
        sys.exit(1)

    for o, a in opts:
        if o in ('-c'):
            createNCPSserver = True
            ncp_server_name = str.strip( a )
        elif o in ('-a'):
            absoluteName = True
            createNCPSserver = True
            ncp_server_name = str.strip( a )
        elif o in ('-i'):
            ncp_ip_address = str.strip( a )
        elif o in ('-v'):
            ncp_volumes = str.split( a, ":" )
        elif o in ('-d'):
            deleteNCPServer = True
            ncp_server_name = str.strip( a )

    ## sanity check
    if not ncp_server_name:
        sys.stderr.write( "NCP server name missing.\n" )
        return( 2 )
    elif not ncp_ip_address and createNCPSserver:
        sys.stderr.write( "An IP address is needed to create a NCP server. \n" )
        return( 3 )

    ss = clstrlibss.load()

    cluster_name = str.split( str.split( ss.clusterDn, "," )[0], "=" )[1]

    if not absoluteName and not ncp_server_name.lower().startswith( str.lower( cluster_name.replace( '_', '-' ) )):
        ncp_server_name = cluster_name.replace( '_', '-' ) + "-" + ncp_server_name

    if not absoluteName and not ncp_server_name.lower().endswith( "server" ):
        ncp_server_name += "-server"

    ld = ldap.initialize( ss.ldapUrl )

    try:
        ld.simple_bind_s( ss.adminDn, ss.adminPw )
    except ldap.LDAPError as e:
        print (e)
        sys.exit( 4 )

    containerDn = clstrlib.containerFromLdapDn(ss.clusterDn)

    ## create NCP server object
    if createNCPSserver:
        ncpServerDn = clstrlib.createVirtualNCPServer( ld, ss.clusterDn, containerDn, ncp_server_name, ncp_ip_address )
        if ncpServerDn:
            print ("NCP Server '" + ncpServerDn + "' created.")
            for vol in ncp_volumes:
                rc = enableNativeVolume( ld, ss.clusterDn, ncpServerDn, containerDn, str.strip( vol ), cluster_name )
                if rc:
                    sys.stderr.write( "Volume '" + vol + "' not found.\n" )
        else:
            sys.stderr.write( "Failed to create NCP server '" + ncp_server_name + "'.\n" )
            rc = 5
    ## delete virtual server object
    elif deleteNCPServer:
        ncpServerDn = "cn=" + ncp_server_name + "," + containerDn
        try:
            ld.delete_s( ncpServerDn )
        except:
            rc = 6

        if rc:
            sys.stderr.write( "Failed to delete NCP server '" + ncp_server_name + "'.\n" )
        else:
            print ("NCP Server '" + ncpServerDn + "' deleted.")

    ld.unbind_s()
    sys.exit(rc)

if __name__ == "__main__":
    sys.exit(main())

